@@ -34,20 +34,14 @@ class RandomBooleanGenerator(BooleanGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
-            'transform': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
+            'fit': {'time': 2e-5, 'memory': 400.0},
+            'transform': {'time': 1e-5, 'memory': 400.0},
             'reverse_transform': {
-                'time': 5e-6,
+                'time': 5e-5,
                 'memory': 500.0,
-            }
+            },
         }
 
 
@@ -69,20 +63,14 @@ class RandomBooleanNaNsGenerator(BooleanGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
-            'transform': {
-                'time': 1e-5,
-                'memory': 1000.0
-            },
+            'fit': {'time': 2e-5, 'memory': 400.0},
+            'transform': {'time': 1e-5, 'memory': 1000.0},
             'reverse_transform': {
                 'time': 5e-5,
                 'memory': 1000.0,
-            }
+            },
         }
 
 
@@ -102,20 +90,14 @@ class RandomSkewedBooleanGenerator(BooleanGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
-            'transform': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
+            'fit': {'time': 1e-5, 'memory': 400.0},
+            'transform': {'time': 1e-5, 'memory': 400.0},
             'reverse_transform': {
-                'time': 5e-6,
+                'time': 5e-5,
                 'memory': 500.0,
-            }
+            },
         }
 
 
@@ -137,20 +119,14 @@ class RandomSkewedBooleanNaNsGenerator(BooleanGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
-            'transform': {
-                'time': 1e-5,
-                'memory': 1000.0
-            },
+            'fit': {'time': 1e-5, 'memory': 400.0},
+            'transform': {'time': 1e-5, 'memory': 1000.0},
             'reverse_transform': {
                 'time': 5e-5,
                 'memory': 1000.0,
-            }
+            },
         }
 
 
@@ -165,20 +141,14 @@ class ConstantBooleanGenerator(BooleanGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
-            'transform': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
+            'fit': {'time': 1e-5, 'memory': 400.0},
+            'transform': {'time': 1e-5, 'memory': 400.0},
             'reverse_transform': {
-                'time': 5e-6,
+                'time': 5e-5,
                 'memory': 500.0,
-            }
+            },
         }
 
 
@@ -199,18 +169,12 @@ class ConstantBooleanNaNsGenerator(BooleanGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 1e-6,
-                'memory': 200.0
-            },
-            'transform': {
-                'time': 1e-5,
-                'memory': 1000.0
-            },
+            'fit': {'time': 1e-5, 'memory': 400.0},
+            'transform': {'time': 1e-5, 'memory': 1000.0},
             'reverse_transform': {
                 'time': 5e-5,
                 'memory': 1000.0,
-            }
+            },
         }
