@@ -26,20 +26,14 @@ class RandomIntegerGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 5e-05,
-                'memory': 400.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 5e-05, 'memory': 400.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 1000.0,
-            }
+            },
         }
 
 
@@ -53,20 +47,14 @@ class RandomIntegerNaNsGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 5e-05,
-                'memory': 1000.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 5e-05, 'memory': 1000.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 1000.0,
-            }
+            },
         }
 
 
@@ -81,20 +69,14 @@ class RandomStringGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 500.0
-            },
-            'transform': {
-                'time': 1e-05,
-                'memory': 500.0
-            },
+            'fit': {'time': 2e-05, 'memory': 500.0},
+            'transform': {'time': 1e-05, 'memory': 500.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 1000.0,
-            }
+            },
         }
 
 
@@ -108,20 +90,14 @@ class RandomStringNaNsGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 1e-05,
-                'memory': 1000.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 1e-05, 'memory': 1000.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 1000.0,
-            }
+            },
         }
 
 
@@ -136,12 +112,13 @@ class RandomMixedGenerator(CategoricalGenerator):
         """Generate a ``num_rows`` number of rows."""
         cat_size = 5
         categories = np.hstack([
-            cat.astype('O') for cat in [
+            cat.astype('O')
+            for cat in [
                 RandomGapDatetimeGenerator.generate(cat_size),
                 np.random.randint(0, 100, cat_size),
                 np.random.uniform(0, 100, cat_size),
                 np.arange(cat_size).astype(str),
-                np.array([True, False])
+                np.array([True, False]),
             ]
         ])
 
@@ -149,20 +126,14 @@ class RandomMixedGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 1e-05,
-                'memory': 1000.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 1e-05, 'memory': 1000.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 2000.0,
-            }
+            },
         }
 
 
@@ -187,20 +158,14 @@ class RandomMixedNaNsGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 1e-05,
-                'memory': 2000.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 1e-05, 'memory': 2000.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 2000.0,
-            }
+            },
         }
 
 
@@ -215,20 +180,14 @@ class SingleIntegerGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 3e-05,
-                'memory': 400.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 3e-05, 'memory': 400.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 400.0,
-            }
+            },
         }
 
 
@@ -242,20 +201,14 @@ class SingleIntegerNaNsGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 3e-05,
-                'memory': 400.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 3e-05, 'memory': 400.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 500.0,
-            }
+            },
         }
 
 
@@ -270,20 +223,14 @@ class SingleStringGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 4e-05,
-                'memory': 400.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 4e-05, 'memory': 400.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 400.0,
-            }
+            },
         }
 
 
@@ -297,20 +244,14 @@ class SingleStringNaNsGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 2e-05,
-                'memory': 400.0
-            },
-            'transform': {
-                'time': 3e-05,
-                'memory': 400.0
-            },
+            'fit': {'time': 2e-05, 'memory': 400.0},
+            'transform': {'time': 3e-05, 'memory': 400.0},
             'reverse_transform': {
                 'time': 1e-05,
                 'memory': 500.0,
-            }
+            },
         }
 
 
@@ -324,20 +265,14 @@ class UniqueIntegerGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 0.0004,
-                'memory': 2000.0
-            },
-            'transform': {
-                'time': 0.0004,
-                'memory': 500000.0
-            },
+            'fit': {'time': 0.0004, 'memory': 2000.0},
+            'transform': {'time': 0.0004, 'memory': 500000.0},
             'reverse_transform': {
-                'time': 0.0004,
+                'time': 0.0005,
                 'memory': 1000000.0,
-            }
+            },
         }
 
 
@@ -351,20 +286,14 @@ class UniqueIntegerNaNsGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 0.0004,
-                'memory': 1000.0
-            },
-            'transform': {
-                'time': 0.0004,
-                'memory': 1000000.0
-            },
+            'fit': {'time': 0.0004, 'memory': 1000.0},
+            'transform': {'time': 0.0004, 'memory': 1000000.0},
             'reverse_transform': {
-                'time': 0.0004,
+                'time': 0.0005,
                 'memory': 1000000.0,
-            }
+            },
         }
 
 
@@ -378,20 +307,14 @@ class UniqueStringGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 0.002,
-                'memory': 2000.0
-            },
-            'transform': {
-                'time': 0.0004,
-                'memory': 500000.0
-            },
+            'fit': {'time': 0.002, 'memory': 2000.0},
+            'transform': {'time': 0.0004, 'memory': 500000.0},
             'reverse_transform': {
-                'time': 0.0004,
+                'time': 0.0005,
                 'memory': 1000000.0,
-            }
+            },
         }
 
 
@@ -405,18 +328,12 @@ class UniqueStringNaNsGenerator(CategoricalGenerator):
 
     @staticmethod
     def get_performance_thresholds():
-        """Return the expected threseholds."""
+        """Return the expected thresholds."""
         return {
-            'fit': {
-                'time': 0.001,
-                'memory': 1000.0
-            },
-            'transform': {
-                'time': 0.0005,
-                'memory': 1000000.0
-            },
+            'fit': {'time': 0.001, 'memory': 1000.0},
+            'transform': {'time': 0.0005, 'memory': 1000000.0},
             'reverse_transform': {
                 'time': 0.0005,
                 'memory': 1000000.0,
-            }
+            },
         }
