@@ -21,19 +21,35 @@ class BinaryEncoder(BaseTransformer):
             replace them with the most common value.
             Defaults to ``mode``.
         model_missing_values (bool):
-            Whether to create a new column to indicate which values were null or not. The column
-            will be created only if there are null values. If ``True``, create the new column if
-            there are null values. If ``False``, do not create the new column even if there
-            are null values. Defaults to ``False``.
+            **DEPRECATED** Whether to create a new column to indicate which values were null or
+            not. The column will be created only if there are null values. If ``True``, create
+            the new column if there are null values. If ``False``, do not create the new column
+            even if there are null values. Defaults to ``False``.
+        missing_value_generation (str or None):
+            The way missing values are being handled. There are three strategies:
+
+                * ``random``: Randomly generates missing values based on the percentage of
+                  missing values.
+                * ``from_column``: Creates a binary column that describes whether the original
+                  value was missing. Then use it to recreate missing values.
+                * ``None``: Do nothing with the missing values on the reverse transform. Simply
+                  pass whatever data we get through.
     """
 
     INPUT_SDTYPE = 'boolean'
     null_transformer = None
 
-    def __init__(self, missing_value_replacement='mode', model_missing_values=False):
+    def __init__(
+        self,
+        missing_value_replacement='mode',
+        model_missing_values=None,
+        missing_value_generation='random',
+    ):
         super().__init__()
-        self._set_missing_value_replacement('mode', missing_value_replacement)
-        self.model_missing_values = model_missing_values
+        self._set_missing_value_generation(missing_value_generation)
+        self._set_missing_value_replacement('random', missing_value_replacement)
+        if model_missing_values is not None:
+            self._set_model_missing_values(model_missing_values)
 
     def _fit(self, data):
         """Fit the transformer to the data.
@@ -43,12 +59,14 @@ class BinaryEncoder(BaseTransformer):
                 Data to fit to.
         """
         self.null_transformer = NullTransformer(
-            self.missing_value_replacement,
-            self.model_missing_values
+            self.missing_value_replacement, self.missing_value_generation
         )
         self.null_transformer.fit(data)
         if self.null_transformer.models_missing_values():
-            self.output_properties['is_null'] = {'sdtype': 'float', 'next_transformer': None}
+            self.output_properties['is_null'] = {
+                'sdtype': 'float',
+                'next_transformer': None,
+            }
 
     def _transform(self, data):
         """Transform boolean to float.
@@ -60,7 +78,7 @@ class BinaryEncoder(BaseTransformer):
             data (pandas.Series):
                 Data to transform.
 
-        Returns
+        Returns:
             np.ndarray
         """
         data = pd.to_numeric(data, errors='coerce')
@@ -92,3 +110,20 @@ class BinaryEncoder(BaseTransformer):
         data[isna] = np.nan
 
         return data
+
+    def _set_fitted_parameters(self, column_name, null_transformer):
+        """Manually set the parameters on the transformer to get it into a fitted state.
+
+        Args:
+            column_name (str):
+                The name of the column to use for the transformer.
+            null_transformer (NullTransformer):
+                A fitted null transformer instance that can be used to generate
+                null values for the column.
+        """
+        self.reset_randomization()
+        self.columns = [column_name]
+        self.output_columns = [column_name]
+        self.null_transformer = null_transformer
+        if self.null_transformer.models_missing_values():
+            self.output_columns.append(column_name + '.is_null')
