@@ -1,14 +1,316 @@
 """Transformers for categorical data."""
 
+import logging
 import warnings
 
 import numpy as np
 import pandas as pd
-import psutil
 from scipy.stats import norm
 
 from rdt.errors import TransformerInputError
 from rdt.transformers.base import BaseTransformer
+from rdt.transformers.utils import (
+    check_nan_in_transform,
+    fill_nan_with_none,
+    try_convert_to_dtype,
+)
+
+LOGGER = logging.getLogger(__name__)
+
+
+class UniformEncoder(BaseTransformer):
+    """Transformer for categorical data.
+
+    This transformer computes a float representative for each one of the categories
+    found in the fit data, and then replaces the instances of these categories with
+    the corresponding representative.
+
+    The representatives are decided by computing the frequencies of each labels and
+    then dividing the ``[0, 1]`` interval according to these frequencies.
+
+    When the transformation is reverted, each value is assigned the category that
+    corresponds to the interval it falls in.
+
+    Null values are considered just another category.
+
+    Args:
+        order_by (str or None):
+            String defining how to order the data before applying the labels. Options are
+            'alphabetical', 'numerical' and ``None``. Defaults to ``None``.
+    """
+
+    INPUT_SDTYPE = 'categorical'
+    SUPPORTED_SDTYPES = ['categorical', 'boolean']
+    frequencies = None
+    intervals = None
+    dtype = None
+
+    def __init__(self, order_by=None):
+        super().__init__()
+        if order_by not in [None, 'alphabetical', 'numerical_value']:
+            raise TransformerInputError(
+                "order_by must be one of the following values: None, 'numerical_value' or "
+                "'alphabetical'"
+            )
+
+        self.order_by = order_by
+
+    def _order_categories(self, unique_data):
+        nans = pd.isna(unique_data)
+        if self.order_by == 'alphabetical':
+            # pylint: disable=invalid-unary-operand-type
+            if any(map(lambda item: not isinstance(item, str), unique_data[~nans])):  # noqa: C417
+                raise TransformerInputError(
+                    "The data must be of type string if order_by is 'alphabetical'."
+                )
+        elif self.order_by == 'numerical_value':
+            if not np.issubdtype(unique_data.dtype.type, np.number):
+                raise TransformerInputError(
+                    "The data must be numerical if order_by is 'numerical_value'."
+                )
+
+        if self.order_by is not None:
+            unique_data = np.sort(unique_data[~nans])  # pylint: disable=invalid-unary-operand-type
+            if nans.any():
+                unique_data = np.append(unique_data, [None])
+
+        return unique_data
+
+    @classmethod
+    def _get_message_unseen_categories(cls, unseen_categories):
+        """Message to raise when there is unseen categories.
+
+        Args:
+            unseen_categories (list): list of unseen categories
+
+        Returns:
+            message to print
+        """
+        categories_to_print = ', '.join(str(x) for x in unseen_categories[:3])
+        if len(unseen_categories) > 3:
+            categories_to_print = f'{categories_to_print}, +{len(unseen_categories) - 3} more'
+
+        return categories_to_print
+
+    @staticmethod
+    def _compute_frequencies_intervals(categories, freq):
+        """Compute the frequencies and intervals of the categories.
+
+        Args:
+            categories (list):
+                List of categories.
+            freq (list):
+                List of frequencies.
+
+        Returns:
+            tuple[dict, dict]:
+                First dict maps categories to their frequency and the
+                second dict maps the categories to their intervals.
+        """
+        frequencies = dict(zip(categories, freq))
+        shift = np.cumsum(np.hstack([0, freq]))
+        shift[-1] = 1
+        list_int = [[shift[i], shift[i + 1]] for i in range(len(shift) - 1)]
+        intervals = dict(zip(categories, list_int))
+
+        return frequencies, intervals
+
+    def _fit(self, data):
+        """Fit the transformer to the data.
+
+        Compute the frequencies of each category and use them
+        to map the column to a numerical one.
+
+        Args:
+            data (pandas.Series):
+                Data to fit the transformer to.
+        """
+        self.dtype = data.dtypes
+        data = fill_nan_with_none(data)
+        labels = pd.unique(data)
+        labels = self._order_categories(labels)
+        freq = data.value_counts(normalize=True, dropna=False)
+        nan_value = freq[np.nan] if np.nan in freq.index else None
+        freq = freq.reindex(labels, fill_value=nan_value).array
+
+        self.frequencies, self.intervals = self._compute_frequencies_intervals(labels, freq)
+
+    def _set_fitted_parameters(self, column_name, intervals, dtype='object'):
+        """Manually set the parameters on the transformer to get it into a fitted state.
+
+        Args:
+            column_name (str):
+                The name of the column for this transformer.
+            intervals (dict[str, tuple]):
+                A dictionary mapping categories to the interval in the range [0, 1]
+                it should map to.
+            dtype (str, optional):
+                The dtype to convert the reverse transformed data back to. Defaults to 'object'.
+        """
+        self.reset_randomization()
+        self.columns = [column_name]
+        self.output_columns = [column_name]
+        self.intervals = intervals
+        self.dtype = dtype
+
+    def _transform(self, data):
+        """Map the category to a continuous value.
+
+        This value is sampled from a uniform distribution
+        with boudaries defined by the frequencies.
+
+        Args:
+            data (pandas.Series):
+                Data to transform.
+
+        Returns:
+            pandas.Series
+        """
+        data_with_none = fill_nan_with_none(data)
+        unseen_indexes = ~(data_with_none.isin(self.frequencies))
+        if unseen_indexes.any():
+            # Keep the 3 first unseen categories
+            unseen_categories = list(data.loc[unseen_indexes].unique())
+            categories_to_print = self._get_message_unseen_categories(unseen_categories)
+            warnings.warn(
+                f"The data in column '{self.get_input_column()}' contains new categories "
+                f"that did not appear during 'fit' ({categories_to_print}). Assigning "
+                'them random values. If you want to model new categories, '
+                "please fit the data again using 'fit'.",
+                category=UserWarning,
+            )
+
+            choices = list(self.frequencies.keys())
+            size = unseen_indexes.size
+            data_with_none[unseen_indexes] = np.random.choice(choices, size=size)
+
+        def map_labels(label):
+            return np.random.uniform(self.intervals[label][0], self.intervals[label][1])
+
+        return data_with_none.map(map_labels).astype(float)
+
+    def _reverse_transform(self, data):
+        """Convert float values back to the original categorical values.
+
+        Args:
+            data (pandas.Series):
+                Data to revert.
+
+        Returns:
+            pandas.Series
+        """
+        check_nan_in_transform(data, self.dtype)
+        data = data.clip(0, 1)
+        bins = [0]
+        labels = []
+        nan_name = 'NaN'
+        while nan_name in self.intervals.keys():
+            nan_name += '_'
+
+        for key, interval in self.intervals.items():
+            bins.append(interval[1])
+            if pd.isna(key):
+                labels.append(nan_name)
+            else:
+                labels.append(key)
+
+        result = pd.cut(data, bins=bins, labels=labels, include_lowest=True)
+        if nan_name in result.cat.categories:
+            result = result.cat.remove_categories(nan_name)
+
+        result = try_convert_to_dtype(result, self.dtype)
+
+        return result
+
+
+class OrderedUniformEncoder(UniformEncoder):
+    """Ordered uniform encoder for categorical data.
+
+    This class works very similarly to the ``UniformEncoder``, except that it requires the ordering
+    for the labels to be provided.
+    Null values are considered just another category.
+
+    Args:
+        order (list):
+            A list of all the unique categories for the data. The order of the list determines the
+            label that each category will get.
+    """
+
+    def __init__(self, order):
+        self.order = fill_nan_with_none(pd.Series(order))
+        if not self.order.is_unique:
+            error_msg = (
+                "The OrderedUniformEncoder has duplicate categories in the 'order' parameter. "
+                'Please drop the duplicates to proceed.'
+            )
+            raise TransformerInputError(error_msg)
+
+        super().__init__()
+
+    def __repr__(self):
+        """Represent initialization of transformer as text.
+
+        Returns:
+            str:
+                The name of the transformer followed by any non-default parameters.
+        """
+        class_name = self.__class__.get_name()
+        custom_args = ['order=<CUSTOM>']
+        args_string = ', '.join(custom_args)
+        return f'{class_name}({args_string})'
+
+    def _check_unknown_categories(self, data):
+        missing = list(data[~data.isin(self.order)].unique())
+        if len(missing) > 0:
+            raise TransformerInputError(
+                f"Unknown categories '{missing}'. All possible categories must be defined in the "
+                "'order' parameter."
+            )
+
+    def _fit(self, data):
+        """Fit the transformer to the data.
+
+        Create all the class attributes while respecting the speicified
+        order of the labels.
+
+        Args:
+            data (pandas.Series):
+                Data to fit the transformer to.
+        """
+        self.dtype = data.dtypes
+        data = fill_nan_with_none(data)
+        self._check_unknown_categories(data)
+
+        category_not_seen = set(self.order.dropna()) != set(data.dropna())
+        nans_not_seen = pd.isna(self.order).any() and not pd.isna(data).any()
+        if category_not_seen or nans_not_seen:
+            unseen_categories = [x for x in self.order if x not in data.array]
+            categories_to_print = self._get_message_unseen_categories(unseen_categories)
+            LOGGER.info(
+                "For column '%s', some of the provided category values were not present in the"
+                ' data during fit: (%s).',
+                self.get_input_column(),
+                categories_to_print,
+            )
+
+            freq = data.value_counts(normalize=True, dropna=False)
+            freq = 0.9 * freq
+            for category in unseen_categories:
+                freq[category] = 0.1 / len(unseen_categories)
+
+        else:
+            freq = data.value_counts(normalize=True, dropna=False)
+
+        nan_value = freq[np.nan] if np.nan in freq.index else None
+        freq = freq.reindex(self.order, fill_value=nan_value).array
+
+        self.frequencies, self.intervals = self._compute_frequencies_intervals(self.order, freq)
+
+    def _transform(self, data):
+        """Map the category to a continuous value."""
+        data = fill_nan_with_none(data)
+        self._check_unknown_categories(data)
+        return super()._transform(data)
 
 
 class FrequencyEncoder(BaseTransformer):
@@ -52,8 +354,14 @@ class FrequencyEncoder(BaseTransformer):
         self.__dict__ = state
 
     def __init__(self, add_noise=False):
+        warnings.warn(
+            "The 'FrequencyEncoder' transformer will no longer be supported in future versions "
+            "of the RDT library. Please use the 'UniformEncoder' transformer instead.",
+            FutureWarning,
+        )
         super().__init__()
         self.add_noise = add_noise
+        self._is_integer = None
 
     @staticmethod
     def _get_intervals(data):
@@ -67,7 +375,7 @@ class FrequencyEncoder(BaseTransformer):
             dict:
                 intervals for each categorical value (start, end).
         """
-        data = data.fillna(np.nan)
+        data = data.infer_objects().fillna(np.nan)
         frequencies = data.value_counts(dropna=False)
         augmented_frequencies = frequencies.to_frame()
         sortable_column_name = f'sortable_{frequencies.name}'
@@ -83,8 +391,7 @@ class FrequencyEncoder(BaseTransformer):
 
         augmented_frequencies[sortable_column_name] = frequencies.index.map(tie_breaker)
         augmented_frequencies = augmented_frequencies.sort_values(
-            [column_name, sortable_column_name],
-            ascending=[False, True]
+            [column_name, sortable_column_name], ascending=[False, True]
         )
         sorted_frequencies = augmented_frequencies[column_name]
 
@@ -139,7 +446,7 @@ class FrequencyEncoder(BaseTransformer):
 
     def _transform_by_category(self, data):
         """Transform the data by iterating over the different categories."""
-        result = np.empty(shape=(len(data), ), dtype=float)
+        result = np.empty(shape=(len(data),), dtype=float)
 
         # loop over categories
         for category, values in self.intervals.items():
@@ -147,13 +454,14 @@ class FrequencyEncoder(BaseTransformer):
             if category is np.nan:
                 mask = data.isna()
             else:
-                mask = (data.to_numpy() == category)
+                mask = data.to_numpy() == category
 
             if self.add_noise:
                 result[mask] = norm.rvs(
-                    mean, std,
+                    mean,
+                    std,
                     size=mask.sum(),
-                    random_state=self.random_states['transform']
+                    random_state=self.random_states['transform'],
                 )
                 result[mask] = self._clip_noised_transform(result[mask], start, end)
             else:
@@ -176,7 +484,9 @@ class FrequencyEncoder(BaseTransformer):
 
     def _transform_by_row(self, data):
         """Transform the data row by row."""
-        return data.fillna(np.nan).apply(self._get_value).to_numpy()
+        data = data.infer_objects().fillna(np.nan).apply(self._get_value).to_numpy()
+
+        return data
 
     def _transform(self, data):
         """Transform the categorical values to float representatives.
@@ -207,27 +517,14 @@ class FrequencyEncoder(BaseTransformer):
 
         return self._transform_by_row(data)
 
-    def _reverse_transform_by_matrix(self, data):
-        """Reverse transform the data with matrix operations."""
-        num_rows = len(data)
-        num_categories = len(self.starts)
-
-        data = np.broadcast_to(data, (num_categories, num_rows)).T
-        starts = np.broadcast_to(self.starts.index, (num_rows, num_categories))
-        is_data_greater_than_starts = (data >= starts)[:, ::-1]
-        interval_indexes = num_categories - np.argmax(is_data_greater_than_starts, axis=1) - 1
-
-        get_category_from_index = list(self.starts.category).__getitem__
-        return pd.Series(interval_indexes).apply(get_category_from_index).astype(self.dtype)
-
     def _reverse_transform_by_category(self, data):
         """Reverse transform the data by iterating over all the categories."""
-        result = np.empty(shape=(len(data), ), dtype=self.dtype)
+        result = np.empty(shape=(len(data),), dtype=self.dtype)
 
         # loop over categories
         for category, values in self.intervals.items():
             start = values[0]
-            mask = (start <= data.to_numpy())
+            mask = start <= data.to_numpy()
             result[mask] = category
 
         return pd.Series(result, index=data.index, dtype=self.dtype)
@@ -250,16 +547,11 @@ class FrequencyEncoder(BaseTransformer):
         Returns:
             pandas.Series
         """
+        check_nan_in_transform(data, self.dtype)
         data = data.clip(0, 1)
         num_rows = len(data)
         num_categories = len(self.means)
 
-        # total shape * float size * number of matrices needed
-        needed_memory = num_rows * num_categories * 8 * 3
-        available_memory = psutil.virtual_memory().available
-        if available_memory > needed_memory:
-            return self._reverse_transform_by_matrix(data)
-
         if num_rows > num_categories:
             return self._reverse_transform_by_category(data)
 
@@ -285,6 +577,7 @@ class OneHotEncoder(BaseTransformer):
     _dummy_encoded = False
     _indexer = None
     _uniques = None
+    dtype = None
 
     @staticmethod
     def _prepare_data(data):
@@ -322,6 +615,7 @@ class OneHotEncoder(BaseTransformer):
             data (pandas.Series or pandas.DataFrame):
                 Data to fit the transformer to.
         """
+        self.dtype = data.dtype
         data = self._prepare_data(data)
 
         null = pd.isna(data).to_numpy()
@@ -331,7 +625,7 @@ class OneHotEncoder(BaseTransformer):
         self._indexer = list(range(self._num_dummies))
         self.dummies = self._uniques.copy()
 
-        if not np.issubdtype(data.dtype, np.number):
+        if not np.issubdtype(data.dtype.type, np.number):
             self._dummy_encoded = True
 
         if self._dummy_na:
@@ -374,7 +668,7 @@ class OneHotEncoder(BaseTransformer):
         """
         data = self._prepare_data(data)
         unique_data = {np.nan if pd.isna(x) else x for x in pd.unique(data)}
-        unseen_categories = unique_data - set(self.dummies)
+        unseen_categories = unique_data - {np.nan if pd.isna(x) else x for x in self.dummies}
         if unseen_categories:
             # Select only the first 5 unseen categories to avoid flooding the console.
             examples_unseen_categories = set(list(unseen_categories)[:5])
@@ -397,6 +691,7 @@ class OneHotEncoder(BaseTransformer):
         Returns:
             pandas.Series
         """
+        check_nan_in_transform(data, self.dtype)
         if not isinstance(data, np.ndarray):
             data = data.to_numpy()
 
@@ -404,8 +699,10 @@ class OneHotEncoder(BaseTransformer):
             data = data.reshape(-1, 1)
 
         indices = np.argmax(data, axis=1)
+        result = pd.Series(indices).map(self.dummies.__getitem__)
+        result = try_convert_to_dtype(result, self.dtype)
 
-        return pd.Series(indices).map(self.dummies.__getitem__)
+        return result
 
 
 class LabelEncoder(BaseTransformer):
@@ -439,6 +736,7 @@ class LabelEncoder(BaseTransformer):
     SUPPORTED_SDTYPES = ['categorical', 'boolean']
     values_to_categories = None
     categories_to_values = None
+    dtype = 'O'
 
     def __init__(self, add_noise=False, order_by=None):
         super().__init__()
@@ -483,12 +781,12 @@ class LabelEncoder(BaseTransformer):
             data (pandas.Series):
                 Data to fit the transformer to.
         """
-        unique_data = pd.unique(data.fillna(np.nan))
+        self.dtype = data.dtype
+        unique_data = pd.unique(data.infer_objects().fillna(np.nan))
         unique_data = self._order_categories(unique_data)
         self.values_to_categories = dict(enumerate(unique_data))
         self.categories_to_values = {
-            category: value
-            for value, category in self.values_to_categories.items()
+            category: value for value, category in self.values_to_categories.items()
         }
 
     def _transform(self, data):
@@ -506,7 +804,7 @@ class LabelEncoder(BaseTransformer):
         Returns:
             pd.Series
         """
-        mapped = data.fillna(np.nan).map(self.categories_to_values)
+        mapped = data.infer_objects().fillna(np.nan).map(self.categories_to_values)
         is_null = mapped.isna()
         if is_null.any():
             # Select only the first 5 unseen categories to avoid flooding the console.
@@ -518,12 +816,10 @@ class LabelEncoder(BaseTransformer):
                 'please fit the transformer again with the new data.'
             )
 
-        mapped[is_null] = np.random.randint(
-            len(self.categories_to_values),
-            size=is_null.sum()
-        )
+        mapped[is_null] = np.random.randint(len(self.categories_to_values), size=is_null.sum())
 
         if self.add_noise:
+            mapped = mapped.astype(float)
             mapped = np.random.uniform(mapped, mapped + 1)
 
         return mapped
@@ -538,14 +834,18 @@ class LabelEncoder(BaseTransformer):
         Returns:
             pandas.Series
         """
+        check_nan_in_transform(data, self.dtype)
         if self.add_noise:
             data = np.floor(data)
 
         data = data.clip(min(self.values_to_categories), max(self.values_to_categories))
-        return data.round().map(self.values_to_categories)
+        data = data.round().map(self.values_to_categories)
+        data = try_convert_to_dtype(data, self.dtype)
+
+        return data
 
 
-class CustomLabelEncoder(LabelEncoder):
+class OrderedLabelEncoder(LabelEncoder):
     """Custom label encoder for categorical data.
 
     This class works very similarly to the ``LabelEncoder``, except that it requires the ordering
@@ -564,6 +864,13 @@ class CustomLabelEncoder(LabelEncoder):
 
     def __init__(self, order, add_noise=False):
         self.order = pd.Series(order).fillna(np.nan)
+        if not self.order.is_unique:
+            err_msg = (
+                "The OrderedLabelEncoder has duplicate categories in the 'order' parameter. "
+                'Please drop the duplicates to proceed.'
+            )
+            raise TransformerInputError(err_msg)
+
         super().__init__(add_noise=add_noise)
 
     def __repr__(self):
@@ -593,7 +900,9 @@ class CustomLabelEncoder(LabelEncoder):
             data (pandas.Series):
                 Data to fit the transformer to.
         """
-        data = data.fillna(np.nan)
+        self.dtype = data.dtype
+        data = data.infer_objects().fillna(np.nan)
+
         missing = list(data[~data.isin(self.order)].unique())
         if len(missing) > 0:
             raise TransformerInputError(
@@ -603,6 +912,20 @@ class CustomLabelEncoder(LabelEncoder):
 
         self.values_to_categories = dict(enumerate(self.order))
         self.categories_to_values = {
-            category: value
-            for value, category in self.values_to_categories.items()
+            category: value for value, category in self.values_to_categories.items()
         }
+
+
+class CustomLabelEncoder(OrderedLabelEncoder):
+    """Deprecated class name for ``OrderedLabelEncoder``.
+
+    Class to ensure backwards compatibility with previous versions of RDT.
+    """
+
+    def __init__(self, order, add_noise=False):
+        warnings.warn(
+            "The 'CustomLabelEncoder' is renamed to 'OrderedLabelEncoder'. Please update the"
+            'name to ensure compatibility with future versions of RDT.',
+            FutureWarning,
+        )
+        super().__init__(order, add_noise)
