@@ -10,7 +10,7 @@ from rdt.errors import TransformerInputError
 LOGGER = logging.getLogger(__name__)
 
 
-class NullTransformer():
+class NullTransformer:
     """Transformer for data that contains Null values.
 
     Args:
@@ -18,22 +18,35 @@ class NullTransformer():
             Indicate what to do with the null values. If an integer, float or string is given,
             replace them with the given value. If the strings ``'mean'`` or ``'mode'`` are given,
             replace them with the corresponding aggregation (``'mean'`` only works for numerical
-            values). If ``None`` is given, do not replace them. Defaults to ``None``.
-        model_missing_values (bool):
-            Whether to create a new column to indicate which values were null or not. The column
-            will be created only if there are null values. If ``True``, create the new column if
-            there are null values. If ``False``, do not create the new column even if there
-            are null values. Defaults to ``False``.
+            values) if ``'random'`` replace each null value with a random value in the data range.
+            If ``None`` is given, do not replace them. Defaults to ``None``.
+        missing_value_generation (str or None):
+            The way missing values are being handled. There are three strategies:
+
+                * ``random``: Randomly generates missing values based on the percentage of
+                  missing values.
+                * ``from_column``: Creates a binary column that describes whether the original
+                  value was missing. Then use it to recreate missing values.
+                * ``None``: Do nothing with the missing values on the reverse transform. Simply
+                  pass whatever data we get through.
     """
 
     nulls = None
-    _model_missing_values = None
+    _missing_value_generation = None
     _missing_value_replacement = None
     _null_percentage = None
 
-    def __init__(self, missing_value_replacement=None, model_missing_values=False):
+    def __init__(self, missing_value_replacement=None, missing_value_generation='random'):
         self._missing_value_replacement = missing_value_replacement
-        self._model_missing_values = model_missing_values
+        if missing_value_generation not in (None, 'from_column', 'random'):
+            raise TransformerInputError(
+                "'missing_value_generation' must be one of the following values: "
+                "None, 'from_column' or 'random'."
+            )
+
+        self._missing_value_generation = missing_value_generation
+        self._min_value = None
+        self._max_value = None
 
     def models_missing_values(self):
         """Indicate whether this transformer creates a null column on transform.
@@ -42,7 +55,7 @@ class NullTransformer():
             bool:
                 Whether a null column is created on transform.
         """
-        return self._model_missing_values
+        return self._missing_value_generation == 'from_column'
 
     def _get_missing_value_replacement(self, data):
         """Get the fill value to use for the given data.
@@ -63,11 +76,13 @@ class NullTransformer():
         if self._missing_value_replacement is None:
             return None
 
-        if self._missing_value_replacement in {'mean', 'mode'} and pd.isna(data).all():
-            raise TransformerInputError(
+        if self._missing_value_replacement in {'mean', 'mode', 'random'} and pd.isna(data).all():
+            msg = (
                 f"'missing_value_replacement' cannot be set to '{self._missing_value_replacement}'"
-                ' when the provided data only contains NaNs.'
+                ' when the provided data only contains NaNs. Using 0 instead.'
             )
+            LOGGER.info(msg)
+            return 0
 
         if self._missing_value_replacement == 'mean':
             return data.mean()
@@ -86,20 +101,39 @@ class NullTransformer():
             data (pandas.Series):
                 Data to transform.
         """
-        null_values = data.isna().to_numpy()
-        self.nulls = null_values.any()
-
         self._missing_value_replacement = self._get_missing_value_replacement(data)
-        if not self.nulls and self._model_missing_values:
-            self._model_missing_values = False
-            guidance_message = (
-                f'Guidance: There are no missing values in column {data.name}. '
-                'Extra column not created.'
-            )
-            LOGGER.info(guidance_message)
+        if self._missing_value_replacement == 'random':
+            self._min_value = data.min()
+            self._max_value = data.max()
 
-        if not self._model_missing_values:
-            self._null_percentage = null_values.sum() / len(data)
+        if self._missing_value_generation is not None:
+            null_values = data.isna().to_numpy()
+            self.nulls = null_values.any()
+
+            if not self.nulls and self.models_missing_values():
+                self._missing_value_generation = None
+                guidance_message = (
+                    f'Guidance: There are no missing values in column {data.name}. '
+                    'Extra column not created.'
+                )
+                LOGGER.info(guidance_message)
+
+            if self._missing_value_generation == 'random':
+                self._null_percentage = null_values.sum() / len(data)
+
+    def _set_fitted_parameters(self, null_ratio):
+        """Manually set the parameters on the transformer to get it into a fitted state.
+
+        Args:
+            null_ratio (float):
+                The fraction of values to replace with null values.
+        """
+        if null_ratio < 0 or null_ratio > 1.0:
+            raise ValueError('null_ratio should be a value between 0 and 1.')
+
+        if null_ratio != 0:
+            self.nulls = True
+            self._null_percentage = null_ratio
 
     def transform(self, data):
         """Replace null values with the indicated ``missing_value_replacement``.
@@ -114,10 +148,16 @@ class NullTransformer():
             numpy.ndarray
         """
         isna = data.isna()
-        if isna.any() and self._missing_value_replacement is not None:
-            data = data.fillna(self._missing_value_replacement)
+        if self._missing_value_replacement == 'random':
+            data_mask = list(
+                np.random.uniform(low=self._min_value, high=self._max_value, size=len(data))
+            )
+            data = data.mask(data.isna(), data_mask)
 
-        if self._model_missing_values:
+        elif isna.any() and self._missing_value_replacement is not None:
+            data = data.infer_objects().fillna(self._missing_value_replacement)
+
+        if self._missing_value_generation == 'from_column':
             return pd.concat([data, isna.astype(np.float64)], axis=1).to_numpy()
 
         return data.to_numpy()
@@ -137,14 +177,14 @@ class NullTransformer():
             pandas.Series
         """
         data = data.copy()
-        if self._model_missing_values:
+        if self._missing_value_generation == 'from_column':
             if self.nulls:
                 isna = data[:, 1] > 0.5
 
             data = data[:, 0]
 
         elif self.nulls:
-            isna = np.random.random((len(data), )) < self._null_percentage
+            isna = np.random.random((len(data),)) < self._null_percentage
 
         data = pd.Series(data)
 
